import { assertEquals } from '@std/assert'
import { describe, it } from '@std/testing/bdd'
import { returnsNext, stub } from '@std/testing/mock'
import { fetchEvents } from './fetch_events.ts'
import { aPageResponse, emptyPageResponse, OnlineTimedEvent, RealTimedEvent } from './providers/jasd/tests/fixtures.ts'

export type ResponsePayload = Record<string, unknown> | Error

export const fakeFetch = (response: ResponsePayload) => {
  return stub(
    globalThis,
    'fetch',
    returnsNext([Promise.resolve(new Response(JSON.stringify(response)))]),
  )
}

describe('fetchEvents', () => {
  it('can make sense of an empty response', async () => {
    using _ = fakeFetch(emptyPageResponse)

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result, {
      data: [],
      totalResults: 0,
    })
  })

  it('can fetch an event', async () => {
    using _ = fakeFetch(aPageResponse([RealTimedEvent], true, true))

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result, {
      data: [
        {
          id: '4610',
          title: 'RealTimedEvent',
          organisationName: 'Allerweltskino e.V.',
          startDate: '2025-03-26T00:00:00+01:00',
          endDate: '2025-03-26T00:00:00+01:00',
          location: {
            city: 'Köln',
            street: 'Bahnhofstr.',
            streetNo: '41',
            zipCode: '50674',
          },
          isRemote: false,
          imageUrl: 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files/activities/act5.jpg',
          eventDetails: {
            description:
              '<p>Jeden Dienstag (bis auf eine Sommer- und Winterpause) zeigt der Verein Allerweltskino e.V. im Kölner Filmtheater Off Broadway einen Nachhaltigkeitsfilm. Das Programm ist im Internet abrufbar (siehe Kasten rechts).</p>',
            externalLink: 'https://www.allerweltskino.de',
            meetingLink: 'https://www.allerweltskino.de',
          },
        },
      ],
      totalResults: 1,
    })
  })

  it('satisfies the requested limit of 2', async () => {
    //be aware the current implementation filters permanent events, so this is more of a hypothetical scenario
    const firstPageResponse = Promise.resolve(
      new Response(JSON.stringify(aPageResponse(
        [[
          OnlineTimedEvent,
          OnlineTimedEvent,
          OnlineTimedEvent,
        ]],
        true,
        true,
        3,
      ))),
    )

    using fetchFake = stub(
      globalThis,
      'fetch',
      returnsNext([firstPageResponse]),
    )

    const result = await fetchEvents({ offset: 0, limit: 2 })

    assertEquals(result.data[0].title, 'OnlineTimedEvent', 'its the OnlineTimedEvent')
    assertEquals(result.data[2].title, 'OnlineTimedEvent', 'its the OnlineTimedEvent')
    assertEquals(fetchFake.calls.length, 1, 'only fetches one page')
  })

  it('returns the totalResults of all potentially fetchable events', async () => {
    const eventsFirstPage = [
      RealTimedEvent,
    ]
    const eventsSecondPage = [
      RealTimedEvent,
    ]
    const totalElements = 2

    const firstPageResponse = Promise.resolve(
      new Response(
        JSON.stringify(aPageResponse([eventsFirstPage], true, false, totalElements)),
      ),
    )
    const secondPageResponse = Promise.resolve(
      new Response(JSON.stringify(aPageResponse([eventsSecondPage], false, false, totalElements))),
    )

    using _ = stub(
      globalThis,
      'fetch',
      returnsNext([firstPageResponse, secondPageResponse]),
    )

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result.totalResults, 2)
  })
})
